import { Type, Context } from './utils';
export declare function generateJsonSchemas(context: Context): {
    entry: string;
    schema: {
        $ref: string;
        definitions: {
            [name: string]: Definition;
        };
    };
}[];
export declare function getAllDefinitions(context: Context): {
    [name: string]: Definition;
};
export declare function getJsonSchemaProperty(memberType: Type, context: Context): Definition;
export declare function getReferencedDefinitions(typeName: string | Definition, definitions: {
    [name: string]: Definition;
}, dependents: string[]): {
    [name: string]: Definition;
};
/**
 * @public
 */
export declare type Definition = NumberDefinition | BooleanDefinition | ObjectDefinition | ArrayDefinition | UndefinedDefinition | StringDefinition | NullDefinition | FileDefinition;
/**
 * @public
 */
export declare type NumberDefinition = {
    type: 'number' | 'integer';
    minimum?: number;
    maximum?: number;
    exclusiveMinimum?: number;
    exclusiveMaximum?: number;
    multipleOf?: number;
    default?: number;
} & CommonDefinition;
/**
 * @public
 */
export declare type BooleanDefinition = {
    type: 'boolean';
    default?: boolean;
} & CommonDefinition;
/**
 * @public
 */
export declare type ObjectDefinition = {
    type: 'object';
    additionalProperties?: Definition | boolean;
    properties?: {
        [name: string]: Definition;
    };
    required?: string[];
    minProperties?: number;
    maxProperties?: number;
    anyOf?: Definition[];
    default?: unknown;
} & CommonDefinition;
/**
 * @public
 */
export declare type ArrayDefinition = {
    type: 'array';
    items: Definition;
    uniqueItems?: boolean;
    minItems?: number;
    maxItems?: number;
    default?: unknown[];
} & CommonDefinition;
/**
 * @public
 */
export declare type UndefinedDefinition = {
    type: undefined;
    $ref?: string;
    anyOf?: Definition[];
    default?: unknown;
} & CommonDefinition;
declare type FileDefinition = {
    type: 'file';
};
interface CommonDefinition {
    enum?: unknown[];
    const?: unknown;
    title?: string;
    description?: string;
}
/**
 * @public
 */
export declare type StringDefinition = {
    type: 'string';
    minLength?: number;
    maxLength?: number;
    pattern?: string;
    default?: string;
} & CommonDefinition;
/**
 * @public
 */
export interface NullDefinition {
    type: 'null';
}
export {};
