"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateOcamlTypes = void 0;
const utils_1 = require("./utils");
function generateOcamlTypes(typeDeclarations) {
    const messages = [];
    for (const typeDeclaration of typeDeclarations) {
        if (typeDeclaration.kind === 'object') {
            messages.push(getOcamlTypeOfObjectDeclaration(typeDeclarations, typeDeclaration));
        }
        else if (typeDeclaration.kind === 'enum') {
            const members = typeDeclaration.members.map(m => `  | ${(0, utils_1.toUpperCase)(m.name)}`).join('\n');
            messages.push(`type ${(0, utils_1.toLowerCase)(typeDeclaration.name)} =\n${members}`);
        }
        else if (typeDeclaration.kind === 'union' && typeDeclaration.objectType) {
            messages.push(getOcamlTypeOfObjectDeclaration(typeDeclarations, { ...typeDeclaration, ...typeDeclaration.objectType }));
        }
    }
    return messages.join('\n\n') + '\n';
}
exports.generateOcamlTypes = generateOcamlTypes;
function getOcamlTypeOfObjectDeclaration(typeDeclarations, objectDeclaration) {
    const members = objectDeclaration.members.map(m => {
        const propertyType = getOcamlTypesProperty(typeDeclarations, m.type);
        if (propertyType && m.name) {
            return `  ${m.name}: ${m.optional ? (0, utils_1.toLowerCase)(propertyType) + ' option' : (0, utils_1.toLowerCase)(propertyType)}`;
        }
        return undefined;
    });
    return `type ${(0, utils_1.toLowerCase)(objectDeclaration.name)} = {
${members.filter(m => m).map(m => m + ';').join('\n')}
}`;
}
function getOcamlTypesProperty(typeDeclarations, memberType) {
    let propertyType = '';
    if (memberType.kind === 'array') {
        const elementPropertyType = getOcamlTypesProperty(typeDeclarations, memberType.type);
        if (elementPropertyType) {
            propertyType = `${(0, utils_1.toLowerCase)(elementPropertyType)} list`;
        }
    }
    else if (memberType.kind === 'enum') {
        propertyType = memberType.name;
    }
    else if (memberType.kind === 'reference') {
        propertyType = getOcamlTypesPropertyOfReference(typeDeclarations, memberType);
    }
    else if (memberType.kind === 'number') {
        if (memberType.type === 'number'
            || memberType.type === 'float'
            || memberType.type === 'double') {
            propertyType = 'float';
        }
        else {
            propertyType = 'int';
        }
    }
    else if (memberType.kind === 'string') {
        propertyType = 'string';
    }
    else if (memberType.kind === 'boolean') {
        propertyType = 'bool';
    }
    return propertyType;
}
function getOcamlTypesPropertyOfReference(typeDeclarations, memberType) {
    const typeDeclaration = typeDeclarations.find(m => m.kind === 'enum' && m.name === memberType.name);
    if (typeDeclaration && typeDeclaration.kind === 'enum' && typeDeclaration.type === 'string') {
        return 'string';
    }
    return memberType.name;
}
